/*
 * Copyright E. Bigeon (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * Socket implementation of GCLC.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc-socket:fr.bigeon.gclc.socket.SocketConsoleApplicationTest.java
 * Created on: Jun 1, 2016
 */
package fr.bigeon.gclc.socket;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.logging.Logger;

import org.junit.Test;

/** Test class for {@link SocketConsoleApplicationShell}
 * 
 * @author Emmanuel Bigeon */
@SuppressWarnings({"static-method", "unused", "javadoc", "nls"})
public class SocketConsoleApplicationTest {

    private static final Logger LOGGER = Logger
            .getLogger(SocketConsoleApplicationTest.class.getName());

    @Test
    public void integrationTest() {
        Thread server;
        try {
            server = TestServer.startServer("bye");
        } catch (IOException e3) {
            assertNull(e3);
            fail("unable to start server");
        }
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e1) {
            e1.printStackTrace();
        }
        final String hostName = "127.0.0.1";
        final int portNumber = 3300;

        try (Socket kkSocket = new Socket(hostName, portNumber);
             PrintWriter out = new PrintWriter(kkSocket.getOutputStream(),
                     true);
             BufferedReader in = new BufferedReader(
                     new InputStreamReader(kkSocket.getInputStream()));) {

            String fromServer;
            int i = -1;
            String[] cmds = {"help", "toto", "test", "bye"};
            while ((fromServer = in.readLine()) != null) {
                i++;
                LOGGER.fine("Server: \n" + fromServer);
                if (fromServer.equals("Bye.")) {
                    break;
                }
                while (fromServer != null && !fromServer.equals("> ")) {
                    fromServer = in.readLine();
                    LOGGER.fine("Server: \n" + fromServer);
                }
                if (fromServer == null) {
                    fail("Null pointer");
                }

                final String fromUser = cmds[i];
                if (fromUser != null) {
                    LOGGER.fine("Client: " + fromUser);
                    out.println(fromUser);
                }
            }
            assertEquals(4, i);
        } catch (final IOException e) {
            e.printStackTrace();
        }

        try (Socket kkSocket = new Socket(hostName, portNumber);
             PrintWriter out = new PrintWriter(kkSocket.getOutputStream(),
                     true);
             BufferedReader in = new BufferedReader(
                     new InputStreamReader(kkSocket.getInputStream()));) {

            String fromServer;
            int i = 0;
            String[] cmds = {"help", "toto", "test",
                    ConsoleTestApplication.EXIT};
            while ((fromServer = in.readLine()) != null) {
                LOGGER.fine("Server: \n" + fromServer);
                while (fromServer != null && !fromServer.equals("> ")) {
                    LOGGER.fine("Server: \n" + fromServer);
                    fromServer = in.readLine();
                }
                if (fromServer == null) {
                    break;
                }
                LOGGER.fine("Server: \n" + fromServer);

                final String fromUser = cmds[i];
                if (fromUser != null) {
                    LOGGER.fine("Client: " + fromUser);
                    out.println(fromUser);
                }
                i++;
            }
            assertEquals(4, i);
        } catch (final IOException e) {
            e.printStackTrace();
        }
        try {
            Thread.sleep(100);
        } catch (InterruptedException e1) {
            e1.printStackTrace();
        }

        try (Socket kkSocket = new Socket(hostName, portNumber);
             PrintWriter out = new PrintWriter(kkSocket.getOutputStream(),
                     true);
             BufferedReader in = new BufferedReader(
                     new InputStreamReader(kkSocket.getInputStream()));) {

            String fromServer;
            int i = 0;
            String[] cmds = {"help", "toto", "test",
                    ConsoleTestApplication.EXIT};
            while ((fromServer = in.readLine()) != null) {
                while (fromServer != null && !fromServer.equals("> ")) {
                    fromServer = in.readLine();
                    LOGGER.fine("Server: \n" + fromServer);
                }
                if (fromServer == null) {
                    break;
                }

                final String fromUser = cmds[i];
                if (fromUser != null) {
                    LOGGER.fine("Client: " + fromUser);
                    out.println(fromUser);
                }
                i++;
            }
            assertEquals(4, i);
        } catch (final IOException e) {
            e.printStackTrace();
        }
        TestServer.closeServer();
        try {
            Thread.sleep(100);
        } catch (InterruptedException e2) {
            e2.printStackTrace();
        }
        try {
            server = TestServer.startServer(true);
        } catch (IOException e2) {
            assertNull(e2);
        }
        try {
            Thread.sleep(100);
        } catch (InterruptedException e1) {
            e1.printStackTrace();
        }

        try (Socket kkSocket = new Socket(hostName, portNumber);
             PrintWriter out = new PrintWriter(kkSocket.getOutputStream(),
                     true);
             BufferedReader in = new BufferedReader(
                     new InputStreamReader(kkSocket.getInputStream()));) {

            String fromServer;
            int i = 0;
            String[] cmds = {"help", "test", "close"};
            while ((fromServer = in.readLine()) != null) {
                assertTrue(i < 1);
                while (fromServer != null && !fromServer.equals("> ") &&
                       !fromServer.equals("See you")) {
                    fromServer = in.readLine();
                    LOGGER.fine("Server: \n" + fromServer);
                }
                if (fromServer == null || fromServer.equals("Bye.") ||
                    fromServer.equals("See you")) {
                    break;
                }

                final String fromUser = cmds[i];
                if (fromUser != null) {
                    LOGGER.fine("Client: " + fromUser);
                    out.println(fromUser);
                }
                i++;
            }
            assertEquals(1, i);
        } catch (final IOException e) {
            e.printStackTrace();
        }
        Thread srv = null;
        try {
            srv = TestServer.getServer();
        } catch (IOException e1) {
            assertNull(e1);
        }
        TestServer.closeServer();
        try {
            srv.join();
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
}
